/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static com.perforce.p4scout.Constants.CHANGENUM;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.JOBID;
import static com.perforce.p4scout.Constants.TYPE;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.AdapterView.OnItemSelectedListener;

/**
 * This is the class that displays the GoTo dialog; a shortcut to specific changes or jobs.<br>
 * This can either be used in generic mode, or you can set the TYPE extra to put it in changelist
 * or job modes.<p>
 * The ServerId extra, ID, is required.
 */
public class GotoDialog extends Activity {
	
	private String serverId, type;
	
	/**
	 * This is the initialisation method for the dialog.<br/>
	 * If a specific mode is set, we do not need to see the spinner.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		
		Window w = getWindow();
		w.requestFeature(Window.FEATURE_LEFT_ICON);
		setContentView(R.layout.gotox);
		w.setFeatureDrawableResource(Window.FEATURE_LEFT_ICON, R.drawable.ic_dialog_menu_generic);
		
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		final Context context = this;
		
		type = (String) extras.get(TYPE);
		serverId = (String) extras.get(ID);

		final EditText gotoX = (EditText) findViewById(R.id.etGotoValue);
		final Spinner sType = (Spinner) findViewById(R.id.spGotoType);
		
		if (type != null) {
			sType.setVisibility(View.GONE);
			setTitle(getString(R.string.titleGoto));
		} else {
			sType.setOnItemSelectedListener(new OnItemSelectedListener() {

				@Override
				public void onItemSelected(AdapterView<?> parent, View view,
						int pos, long id) {
					// Set the type, so we know what to GoTo later.
					type = parent.getItemAtPosition(pos).toString();
				}

				@Override
				public void onNothingSelected(AdapterView<?> arg0) {
					// Do nothing
				}
				
			});
		}
			
		((Button) findViewById(R.id.btGo)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (type.contentEquals(getString(R.string.optGotoJob))) {
					// GoTo the job
					Intent jobIntent = new Intent(context, JobDetail.class);
		    		jobIntent.putExtra(ID, serverId);
		    		jobIntent.putExtra(JOBID, gotoX.getText().toString().trim());			    		
		    		startActivity(jobIntent);
				} else if (type.contentEquals(getString(R.string.optGotoChange))) {
					// GoTo the changelist (which must be an integer)
					try  {
						Intent changeIntent = new Intent(context, ChangeDetail.class);
			    		changeIntent.putExtra(ID, serverId);
			    		changeIntent.putExtra(CHANGENUM, Integer.parseInt(gotoX.getText().toString()));			    		
			    		startActivity(changeIntent);
					} catch (NumberFormatException nfe) {
						// That's OK, we can ignore it!
					}
				}
				// Close the dialog behind us.
				finish();
			}	
		});
	}
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
