/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import android.app.Activity;
import android.content.res.Configuration;
import android.content.res.TypedArray;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.AdapterView.OnItemClickListener;

/**
 * The HelpDialog class is displays help topics as a list. Clicking on a topic will expand it
 * to show the topic detail.
 */
public class HelpDialog extends Activity {
	
	protected HelpAdapter helpTopics;
	
	/** 
	 * The help topics are loaded from arrays.xml and loaded into the special list-adapter.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.helplist);

		TypedArray names = getResources().obtainTypedArray(R.array.HelpNameList);
		TypedArray views = getResources().obtainTypedArray(R.array.HelpViewList);
		
		helpTopics = new HelpAdapter(this);
		ListView helpTopicContainer = (ListView) findViewById(R.id.lvHelpTopics);
		helpTopicContainer.setAdapter(helpTopics);
		
		helpTopicContainer.setOnItemClickListener(new OnItemClickListener(){
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				helpTopics.selectTopic(position);
				helpTopics.notifyDataSetChanged();				
			}
		});
		
		for (int i=0; i<names.length(); i++) {
			helpTopics.addTopic(names.getResourceId(i, -1), views.getResourceId(i, -1));
		}
	}
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }

}
