/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import java.util.LinkedList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.perforce.p4java.core.IJob;

/**
 * This is the ListAdaptor implementation for displaying lists of Perforce Jobs.
 * 
 * @see android.widget.BaseAdapter
 */
public class JobListAdapter extends BaseAdapter {

	protected List<IJob> jobs = new LinkedList<IJob>();
    protected LayoutInflater inflater;
    protected Context context;
    
    /**
     * Instantiates a new list adapter for Jobs.
     *
     * @param context parent class' context
     */
    public JobListAdapter(Context context) {
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }   
    
    /**
     * Adds the job to the list.
     *
     * @param job the job object.
     */
    public void addJob(IJob job) {
    	jobs.add(job);
    }
    
    /**
     * Clears the list of jobs.
     */
    public void clear() {
    	jobs.clear();
    }
    
	/* (non-Javadoc)
	 * @see android.widget.Adapter#getCount()
	 */
	public int getCount() {
		return jobs.size();
	}
	
	/* (non-Javadoc)
	 * @see android.widget.Adapter#getItem(int)
	 */
	public IJob getItem(int position) {
		return jobs.get(position);
	}

	/* (non-Javadoc)
	 * @see android.widget.Adapter#getItemId(int)
	 */
	public long getItemId(int position) {
		return position;
	}

	/**
	 * This draws the job listing item at the specified position.
	 * 
	 * @param position The position of the list item to be drawn 
	 * @param convertView 
	 * @param parent
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
	public View getView(int position, View convertView, ViewGroup parent) {
		if (convertView == null) {
		    convertView = inflater.inflate(R.layout.joblisting, parent, false);
		}
		
		((TextView) convertView.findViewById(R.id.tvJobName))
			.setText(jobs.get(position).getId());
		((TextView) convertView.findViewById(R.id.tvJobStatus))
			.setText((String) jobs.get(position).getRawFields().get("Status"));
		((TextView) convertView.findViewById(R.id.tvShortDesc))
			.setText(jobs.get(position).getDescription().split("\n")[0]);
		
		return convertView;
	}

	/**
	 * Refresh the dataset.
	 */
	public void refresh() {
    	notifyDataSetChanged();
    }

}
