/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static android.provider.BaseColumns._ID;
import static com.perforce.p4scout.Constants.ACTION;
import static com.perforce.p4scout.Constants.DESC;
import static com.perforce.p4scout.Constants.EDIT;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.NAME;
import static com.perforce.p4scout.Constants.P4PASS;
import static com.perforce.p4scout.Constants.P4PORT;
import static com.perforce.p4scout.Constants.P4SERVERS;
import static com.perforce.p4scout.Constants.P4USER;
import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

/**
 * This is the class that displays the Add/Edit Perforce Server view.<br>
 * The ServerId extra, ID, is only required if the ACTION extra is set to EDIT.
 */
public class ModifyServer extends Activity implements OnClickListener {

	protected enum Action {
	    ADD, EDIT;
    }
    private P4Data storedServers;
    private Action task;
    private String serverId = null;
    private SharedPreferences sp;
	private int page = 1;
	private Context context;
	
	/** The Handler for updating the job details in the View */
	private Handler errorHandle;
    
	/**
	 * Sanity checks the hostname/port fields
	 */
	private int sanityCheck(Action mode) {
		
		String serverAddr = null;
		String serverPort = null;
		
		if (mode == Action.ADD) {
			serverAddr = ((EditText) this.findViewById(R.id.etServerAddr))
				.getText().toString().trim();
		    
		    serverPort = ((EditText) this.findViewById(R.id.etServerPortNo))
				.getText().toString().trim();
		} else {
			serverAddr = ((EditText) this.findViewById(R.id.etServerPort))
				.getText().toString().trim();
		}
		
	    if (serverAddr.length() <= 0) {
	    	if (mode == Action.ADD) {
	    		serverAddr = ((EditText) this.findViewById(R.id.etServerAddr)).getHint().toString();
	    		((EditText) this.findViewById(R.id.etServerAddr)).setText(serverAddr);
	    	} else {
		    	Toast.makeText(this, R.string.errNoServerSet, Toast.LENGTH_LONG).show();
		    	page = 1;
				((EditText) findViewById(R.id.etServerAddr)).requestFocus();
				((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
				((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
				((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
				((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
				((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
				return 1;
	    	}
	    }
	    
	    if (serverPort == null || serverPort.length() <= 0) {
	    	if (serverAddr.contains(":")) {
	    		serverPort = serverAddr.split(":")[1];
	    		serverAddr = serverAddr.split(":")[0];
	    		if (mode == Action.ADD) {
		    		((EditText) this.findViewById(R.id.etServerAddr)).setText(serverAddr);
		    		((EditText) this.findViewById(R.id.etServerPortNo)).setText(serverPort);
	    		}
	    	} else if (mode == Action.ADD) {
	    		serverPort = ((EditText) this.findViewById(R.id.etServerPortNo)).getHint().toString();
	    		((EditText) this.findViewById(R.id.etServerPortNo)).setText(serverPort);
	    	} else {
	    		Toast.makeText(this, R.string.errNoPortSet, Toast.LENGTH_LONG).show();
	    		if (mode == Action.ADD) {
			    	page = 1;
					((EditText) findViewById(R.id.etServerPortNo)).requestFocus();
					((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
					((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
					((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
					((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
					((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
	    		}
				return 1;
	    	}
	    }
	    try {
			if (Integer.parseInt(serverPort) < 0 || Integer.parseInt(serverPort) >= 65536) {
				Toast.makeText(this, R.string.errPortOutOfRange, Toast.LENGTH_LONG).show();
				if (mode == Action.ADD) {
			    	page = 1;
					((EditText) findViewById(R.id.etServerPortNo)).requestFocus();
					((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
					((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
					((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
					((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
					((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
				}
				return 1;
			}
		} catch (NumberFormatException ne) {
			Toast.makeText(this, R.string.errPortNotInt, Toast.LENGTH_LONG).show();
			if (mode == Action.ADD) {
		    	page = 1;
				((EditText) findViewById(R.id.etServerPortNo)).requestFocus();
				((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
				((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
				((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
				((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
				((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
			}
			return 1;
		}
	    return 0;
	}
	
    /**
     * Adds the Perforce Server to the database.
     * @return 
     */
    private Boolean addServer() {
	    ContentValues values = new ContentValues();
	    
	    String serverName = ((EditText) this.findViewById(R.id.etServerName))
	    		.getText().toString().trim();
	    
	    String serverAddr = ((EditText) this.findViewById(R.id.etServerAddr))
			.getText().toString().trim();
	    
	    String serverPort = ((EditText) this.findViewById(R.id.etServerPortNo))
			.getText().toString().trim();
	    
	    if (serverName.length() <= 0) {
	    	serverName = dupCheck(serverAddr, true); 
	    } else {
	    	if (dupCheck(serverName, false) == null) {
	    		Toast.makeText(this, R.string.errDupName, Toast.LENGTH_LONG).show();
	    		return false;
	    	}
	    }
	    values.put(NAME, serverName);
	        	
	    values.put(DESC, ((EditText) this.findViewById(R.id.etServerDesc))
	    		.getText().toString().trim());
	    values.put(P4PORT, serverAddr + ":" + serverPort);
	    values.put(P4PASS, ((EditText) this.findViewById(R.id.etServerPass))
	    		.getText().toString().trim());
	    
	    EditText ServerUser = (EditText) this.findViewById(R.id.etServerUser);
	    if (ServerUser.getText().toString().trim().length() > 0) {
	    	values.put(P4USER, ServerUser.getText().toString().trim());
	    } else {
	    	values.put(P4USER, ServerUser.getHint().toString().trim());
	    }

    	SQLiteDatabase db = storedServers.getWritableDatabase();
	    db.insertOrThrow(P4SERVERS, null, values);
	    
	    db.close();
	    finish();
	    return true;
    }

    /**
	 * Calls the appropriate action: addServer or updateServer
	 * 
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.btAddServer:
			if (task == Action.ADD) {
				if (addServer()) {
					finish();
				}
			} else if (task == Action.EDIT) {
		    	if (sanityCheck(Action.EDIT) == 0) {
					if (updateServer()) {
						finish();
					}
		    	}
			}
			break;
		}
		
	}
    
    /**
     * This is the initialisation method for this dialog.<br/>
     * We setup the UI as either an add or edit dialog.
     * 
     * @see android.app.Activity#onCreate(android.os.Bundle)
     */
    @Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		storedServers = new P4Data(this);

		Intent intent = this.getIntent();
		Bundle extras = intent.getExtras();
		context = this;
		errorHandle = new Handler(Looper.getMainLooper());
		
		if (extras != null && extras.get(ACTION) != null &&
				((String) extras.get(ACTION)).contentEquals(EDIT)) {
			
			setContentView(R.layout.modifyserver);
			
			Button addServerButton = (Button) this.findViewById(R.id.btAddServer);
			
			addServerButton.setOnClickListener(this);
			sp = PreferenceManager.getDefaultSharedPreferences(this);
			((EditText) this.findViewById(R.id.etServerUser))
					.setHint(sp.getString("prefDefaultUser", "nouser"));
			
			task = Action.EDIT;
			serverId = (String) extras.get(ID);
			setTitle(R.string.titleEditServer);
			addServerButton.setText(R.string.btEditServer);
			populateForm();
		} else {
			setContentView(R.layout.addserver);
			task = Action.ADD;
			Button backButton = (Button) this.findViewById(R.id.btBack);
			Button nextButton = (Button) this.findViewById(R.id.btNext);
			
			sp = PreferenceManager.getDefaultSharedPreferences(this);
			((EditText) this.findViewById(R.id.etServerUser))
					.setHint(sp.getString("prefDefaultUser", "nouser"));
			
			nextButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					switch (page) {
					case 1:
						if (sanityCheck(Action.ADD) != 0) {
							break;
						}						
						page = 2;
						((EditText) findViewById(R.id.etServerUser)).requestFocus();
						((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.GONE);
						((Button) findViewById(R.id.btBack)).setVisibility(View.VISIBLE);
						((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.VISIBLE);
						break;
					case 2:
						page = 3;

						((EditText) findViewById(R.id.etServerName)).requestFocus();
						((EditText) findViewById(R.id.etServerName)).setHint(
								((EditText) findViewById(R.id.etServerAddr)).getText().toString().trim());
						((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
						((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.VISIBLE);
						((Button) findViewById(R.id.btNext)).setText(R.string.btAddServer);
						break;
					case 3:
						if (sanityCheck(Action.ADD) == 0) {
							addServer();
						}
						break;
					}
				}
			});
			
			backButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					switch (page) {
					case 2:
						page = 1;
						((EditText) findViewById(R.id.etServerAddr)).requestFocus();
						((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
						((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
						((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
						break;
					case 3:
						page = 2;
						((EditText) findViewById(R.id.etServerUser)).requestFocus();
						((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
						((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.VISIBLE);
						((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
						break;
					}
				}
			});
			
			Button testServerButton = (Button) findViewById(R.id.btTestServer);
			Button testAuthButton = (Button) findViewById(R.id.btTestAuth);
			
			testServerButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					// Before we actually run the check, lets make sure we have sane values
					if (sanityCheck(Action.ADD) != 0) {
						return;
					}

					((Button) v).setEnabled(false);
					((Button) v).setText(R.string.btTestingServer);
					
					class Worker extends Thread {
					    @Override
						public void run() {
							try {
								if (PerforceSession.PerforceTest(context, ((EditText) findViewById(R.id.etServerAddr))
										.getText().toString().trim() + ":" + ((EditText) findViewById(R.id.etServerPortNo))
										.getText().toString().trim(), null, null) == 0) {
									errorHandle.post(new Runnable() {
										@Override
										public void run() {
												Toast.makeText(context, R.string.ttServerValid,
														Toast.LENGTH_LONG).show();
										}
									});
								}
							} catch (final Exception e) {
								errorHandle.post(new Runnable() {
									@Override
									public void run() {
										Toast.makeText(context, e.getMessage().trim(),
												Toast.LENGTH_LONG).show();
									}
								});
							}
							errorHandle.post(new Runnable() {
								@Override
								public void run() {
									((Button) findViewById(R.id.btTestServer)).setEnabled(true);
									((Button) findViewById(R.id.btTestServer)).setText(R.string.btTestServer);
								}
							});
					    }
					}
					
					Thread tmpThread = new Worker();
					tmpThread.start();
					
				}
			});
			
			testAuthButton.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					// Before we actually run the check, lets make sure we have sane values
					if (sanityCheck(Action.ADD) != 0) {
						return;
					}

					((Button) v).setEnabled(false);
					((Button) v).setText(R.string.btTestingAuth);
					
					class Worker extends Thread {
					    @Override
						public void run() {
							try {
								String user = ((EditText) findViewById(R.id.etServerUser)).getText().toString().trim();
								if (user.length() <= 0) {
									user =((EditText) findViewById(R.id.etServerUser)).getHint().toString().trim();
								}
								
								if (PerforceSession.PerforceTest(context, ((EditText) findViewById(R.id.etServerAddr))
										.getText().toString().trim() + ":" + ((EditText) findViewById(R.id.etServerPortNo))
										.getText().toString().trim(), user, ((EditText) findViewById(R.id.etServerPass))
										.getText().toString().trim()) == 0) {
									errorHandle.post(new Runnable() {
										@Override
										public void run() {
												Toast.makeText(context, R.string.ttAuthValid,
														Toast.LENGTH_LONG).show();
										}
									});
								} else {
									errorHandle.post(new Runnable() {
										@Override
										public void run() {
											Toast.makeText(context, R.string.ttPassRequired,
													Toast.LENGTH_LONG).show();
										}
									});
								}
							} catch ( Exception e) {
								errorHandle.post(new Runnable() {
									@Override
									public void run() {
										Toast.makeText(context, R.string.ttAuthFailed,
												Toast.LENGTH_LONG).show();
									}
								});
							}
							errorHandle.post(new Runnable() {
								@Override
								public void run() {
									((Button) findViewById(R.id.btTestAuth)).setEnabled(true);
									((Button) findViewById(R.id.btTestAuth)).setText(R.string.btTestAuth);
								}
							});
					    }
					}
					
					Thread tmpThread = new Worker();
					tmpThread.start();
					
				}
			});
		}
	}
    
    /**
     * Overrides the back button behaviour when in add mode.
     * This prevents users from hitting back and loosing data entered.
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if ((keyCode == KeyEvent.KEYCODE_BACK)) {
	    	switch (page) {
	    	case 1:
	    		return super.onKeyDown(keyCode, event);
	    	case 2:
	    		page = 1;
				((EditText) findViewById(R.id.etServerAddr)).requestFocus();
				((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.GONE);
				((Button) findViewById(R.id.btBack)).setVisibility(View.INVISIBLE);
				((LinearLayout) findViewById(R.id.llAddServer1)).setVisibility(View.VISIBLE);
				break;
			case 3:
				page = 2;
				((EditText) findViewById(R.id.etServerUser)).requestFocus();
				((LinearLayout) findViewById(R.id.llAddServer3)).setVisibility(View.GONE);
				((LinearLayout) findViewById(R.id.llAddServer2)).setVisibility(View.VISIBLE);
				((Button) findViewById(R.id.btNext)).setText(R.string.btNext);
				break;
	    	}
	    	return true;
    	}
        return super.onKeyDown(keyCode, event);
    }
    
    /**
     * Populates the form with data from the database.
     */
    private void populateForm() {
    	SQLiteDatabase db = storedServers.getWritableDatabase();
    	String[] FROM = { _ID, NAME, DESC, P4PORT, P4USER, P4PASS };
        String ORDER_BY = NAME ;
        Cursor cursor = db.query(P4SERVERS, FROM, _ID + "=?", new String[] {serverId},
        		null, null, ORDER_BY);
	    startManagingCursor(cursor);
	    int serverName = cursor.getColumnIndex(NAME);
	    int serverDesc = cursor.getColumnIndex(DESC);
	    int serverPort = cursor.getColumnIndex(P4PORT);
	    int serverUser = cursor.getColumnIndexOrThrow(P4USER);
	    int serverPasswd = cursor.getColumnIndexOrThrow(P4PASS);
	    
    	EditText ServerName = (EditText) this.findViewById(R.id.etServerName);
	    EditText ServerDesc = (EditText) this.findViewById(R.id.etServerDesc);
	    EditText ServerPort = (EditText) this.findViewById(R.id.etServerPort);
	    EditText ServerUser = (EditText) this.findViewById(R.id.etServerUser);
	    EditText ServerPass = (EditText) this.findViewById(R.id.etServerPass);
    	
	    while (cursor.moveToNext()) {
		    ServerName.setText(cursor.getString(serverName));
	    	setTitle(getString(R.string.titleEditServer) + ": " + cursor.getString(serverName));
		    ServerDesc.setText(cursor.getString(serverDesc));
		    ServerPort.setText(cursor.getString(serverPort));
		    ServerUser.setText(cursor.getString(serverUser));
		    ServerPass.setText(cursor.getString(serverPasswd));
	    }
	    cursor.close();
	    stopManagingCursor(cursor);
	    db.close();	    
    }

    /**
     * Checks that the Server name does not already exist
     */
    private String dupCheck(String name, Boolean rename) {
    	SQLiteDatabase db = storedServers.getWritableDatabase();
    	String[] FROM = { _ID, NAME };
        String ORDER_BY = NAME ;
        Cursor cursor = null;
        
        
        // Nothing to do if the server being edited hasn't changed it's name
    	if (serverId != null) {
    		cursor = db.query(P4SERVERS, FROM, _ID + "=?", new String[] {serverId},
	        		null, null, ORDER_BY);
		    startManagingCursor(cursor);
		    int namePos = cursor.getColumnIndex(NAME);
		    while (cursor.moveToNext()) {
		    	if (cursor.getString(namePos).contentEquals(name)) {
		    		cursor.close();  
		    		db.close();
		    		return name;
		    	}
		    }
    		cursor.close();
    	    stopManagingCursor(cursor);
    	}
    	
        
        String goodName = null;
        int mod = 0;
        do {
        	String tmpName = name;
        	if (mod > 0) {
				tmpName += "." + String.valueOf(mod);
        	}
	        cursor = db.query(P4SERVERS, FROM, NAME + "=?", new String[] {tmpName},
	        		null, null, ORDER_BY);
		    startManagingCursor(cursor);
		    
		    if (cursor.getCount() > 0 && !rename) {
		    	cursor.close();
		    	return null;
		    } else if (cursor.getCount() > 0) {
		    	mod++;
		    } else {
		    	goodName = tmpName;
		    }
	    
		    cursor.close();
		    stopManagingCursor(cursor);
        } while (goodName == null);  
	    
	    db.close();
		return goodName;
    }
    
	/**
     * Updates the Perforce Server in the database.
	 * @return 
     */
    private Boolean updateServer() {
    	if (dupCheck(((EditText) this.findViewById(R.id.etServerName))
	    		.getText().toString().trim(), false) == null) {
    		Toast.makeText(this, R.string.errDupName, Toast.LENGTH_LONG).show();
    		return false;
    	}
    	
    	SQLiteDatabase db = storedServers.getWritableDatabase();
	    ContentValues values = new ContentValues();
	    
	    values.put(NAME, ((EditText) this.findViewById(R.id.etServerName))
	    		.getText().toString().trim());
	    values.put(DESC, ((EditText) this.findViewById(R.id.etServerDesc))
	    		.getText().toString().trim());
	    values.put(P4PORT, ((EditText) this.findViewById(R.id.etServerPort))
	    		.getText().toString().trim());
	    values.put(P4PASS, ((EditText) this.findViewById(R.id.etServerPass))
	    		.getText().toString().trim());
	    
	    EditText ServerUser = (EditText) this.findViewById(R.id.etServerUser);
	    if (ServerUser.getText().toString().trim().length() > 0) {
	    	values.put(P4USER, ServerUser.getText().toString().trim());
	    } else {
	    	values.put(P4USER, ServerUser.getHint().toString().trim());
	    }
	    

	    db.update(P4SERVERS, values, _ID + "=?", new String[] {serverId});
	    
	    db.close();
		return true;
    }
    
    /**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
