/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static android.provider.BaseColumns._ID;
import static com.perforce.p4scout.Constants.DESC;
import static com.perforce.p4scout.Constants.GETCHANGES;
import static com.perforce.p4scout.Constants.LTHRESH;
import static com.perforce.p4scout.Constants.MTHRESH;
import static com.perforce.p4scout.Constants.NAME;
import static com.perforce.p4scout.Constants.P4GLOBALS;
import static com.perforce.p4scout.Constants.P4PASS;
import static com.perforce.p4scout.Constants.P4PORT;
import static com.perforce.p4scout.Constants.P4SERVERS;
import static com.perforce.p4scout.Constants.P4USER;
import static com.perforce.p4scout.Constants.USERNAME;
import static com.perforce.p4scout.Constants.UTHRESH;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * This is the database setup class.
 */
public class P4Data extends SQLiteOpenHelper {

	private static final String DATABASE_NAME = "p4data.db";
	private static final int DATABASE_VERSION = 5;
	
	/**
	 * Instantiates a new data handler object.
	 *
	 * @param ctx the parent class' context.
	 */
	public P4Data(Context ctx) { 
		super(ctx, DATABASE_NAME, null, DATABASE_VERSION);
	}
	
	/**
	 * This method creates the database if it does not already exist.
	 * 
	 * @see android.database.sqlite.SQLiteOpenHelper#onCreate(android.database.sqlite.SQLiteDatabase)
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		
		db.execSQL("CREATE TABLE " + P4SERVERS + " (" +
			_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
			NAME + " TEXT," + 
			DESC + " TEXT NOT NULL, " + 
			P4PORT + " TEXT NOT NULL, " +
			P4USER + " TEXT, " +
			P4PASS + " TEXT);");
		
		db.execSQL("CREATE TABLE " + P4GLOBALS + " (" +
			_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
			USERNAME + " TEXT NOT NULL," + 
			UTHRESH + " INTEGER NOT NULL, " + 
			MTHRESH + " INTEGER NOT NULL, " +
			LTHRESH + " INTEGER NOT NULL, " +
			GETCHANGES + " INTEGER NOT NULL);");
		
		db.execSQL("INSERT INTO " + P4GLOBALS + " (" + USERNAME + ", " +
			UTHRESH + ", " + MTHRESH + ", " + LTHRESH + ", " +
			GETCHANGES + ") VALUES ('nouser', 5, 120, 30, 10);");

	
	}
	
	/**
	 * If the database does exist, but the user schema counter does not match the DATABASE_VERSION
	 * number,<br/>
	 * we need to upgrade the schema. We do this on a per-upgrade counter level basis.<p>
	 * 
	 * If you are at 3, you need to the the upgrade to 4 and the upgrade to 5. If you are at 4, you
	 * only need to upgrade to 5.
	 * 
	 * @see android.database.sqlite.SQLiteOpenHelper#onUpgrade(android.database.sqlite.SQLiteDatabase, int, int)
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		if (oldVersion < 4)
		{
			db.execSQL("ALTER TABLE " + P4SERVERS + " ADD COLUMN " + P4USER + " TEXT;");
			db.execSQL("ALTER TABLE " + P4SERVERS + " ADD COLUMN " + P4PASS + " TEXT;");
		}
		if (oldVersion < 5)
		{
			db.execSQL("CREATE TABLE " + P4GLOBALS + " (" +
				_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
				USERNAME + " TEXT NOT NULL," + 
				UTHRESH + " INTEGER NOT NULL, " + 
				MTHRESH + " INTEGER NOT NULL, " +
				LTHRESH + " INTEGER NOT NULL, " +
				GETCHANGES + " INTEGER NOT NULL);");
			
			db.execSQL("INSERT INTO " + P4GLOBALS + " (" + USERNAME + ", " +
				UTHRESH + ", " + MTHRESH + ", " + LTHRESH + ", " +
				GETCHANGES + ") VALUES ('nouser', 5, 120, 30, 10);");
		}
	}
}